@@ -31,6 +31,16 @@ LINK_RETRY = 'retry'
 # This key denotes the link was created due to symbol constraints and the
 # value will be a set of names that the constraint ensures are satisfied.
 LINK_REASONS = 'reasons'
+#
+# This key denotes a callable that will determine if the target is visited.
+LINK_DECIDER = 'decider'
+
+# Chop off full module names of patterns that are built-in to taskflow...
+_CHOP_PAT = "taskflow.patterns."
+_CHOP_PAT_LEN = len(_CHOP_PAT)
+
+# This key denotes the depth the decider will apply (defaulting to all).
+LINK_DECIDER_DEPTH = 'decider_depth'
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -95,10 +105,20 @@ class Flow(object):
             * ``meta`` is link metadata, a dictionary.
         """
 
+    @abc.abstractmethod
+    def iter_nodes(self):
+        """Iterate over nodes of the flow.
+
+        Iterates over 2-tuples ``(A, meta)``, where
+            * ``A`` is a child (atom or subflow) of current flow;
+            * ``meta`` is link metadata, a dictionary.
+        """
+
     def __str__(self):
-        lines = ["%s: %s" % (reflection.get_class_name(self), self.name)]
-        lines.append("%s" % (len(self)))
-        return "; ".join(lines)
+        cls_name = reflection.get_class_name(self)
+        if cls_name.startswith(_CHOP_PAT):
+            cls_name = cls_name[_CHOP_PAT_LEN:]
+        return "%s: %s(len=%d)" % (cls_name, self.name, len(self))
 
     @property
     def provides(self):
