@@ -29,6 +29,7 @@ from oslo_utils import strutils
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from redis import exceptions as redis_exceptions
+from redis import sentinel
 import six
 from six.moves import range as compat_range
 
@@ -567,7 +568,19 @@ return cmsgpack.pack(result)
                     client_conf[key] = value_type_converter(conf[key])
                 else:
                     client_conf[key] = conf[key]
-        return ru.RedisClient(**client_conf)
+        if conf.get('sentinel') is not None:
+            sentinel_conf = {}
+            # sentinel do not have ssl kwargs
+            for key in client_conf:
+                if 'ssl' not in key:
+                    sentinel_conf[key] = client_conf[key]
+            s = sentinel.Sentinel([(sentinel_conf.pop('host'),
+                                    sentinel_conf.pop('port'))],
+                                  sentinel_kwargs=conf.get('sentinel_kwargs'),
+                                  **sentinel_conf)
+            return s.master_for(conf['sentinel'])
+        else:
+            return ru.RedisClient(**client_conf)
 
     def __init__(self, name, conf,
                  client=None, persistence=None):
@@ -813,11 +826,14 @@ return cmsgpack.pack(result)
                 job_name = job_data['name']
                 job_sequence_id = job_data['sequence']
                 job_details = job_data.get('details', {})
-            except (ValueError, TypeError, KeyError):
+            except (ValueError, TypeError, KeyError, exc.JobFailure):
                 with excutils.save_and_reraise_exception():
                     LOG.warning("Incorrectly formatted job data found at"
                                 " key: %s[%s]", self.listings_key,
                                 raw_job_key, exc_info=True)
+                    LOG.info("Deleting invalid job data at key: %s[%s]",
+                             self.listings_key, raw_job_key)
+                    self._client.hdel(self.listings_key, raw_job_key)
             else:
                 postings.append(RedisJob(self, job_name, job_sequence_id,
                                          raw_job_key, uuid=job_uuid,
