@@ -14,13 +14,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from __future__ import absolute_import
-
 import collections
 import logging
+from unittest import mock
 
 import fixtures
-import mock
 from oslotest import base
 import six
 
@@ -184,7 +182,7 @@ class TestCase(base.BaseTestCase):
         except exceptions.WrappedFailure as e:
             self.assertThat(e, FailureRegexpMatcher(exc_class, pattern))
 
-    def assertItemsEqual(self, seq1, seq2, msg=None):
+    def assertCountEqual(self, seq1, seq2, msg=None):
         matcher = ItemsEqual(seq1)
         self.assertThat(seq2, matcher)
 
