@@ -15,16 +15,7 @@
 #    under the License.
 
 import collections
-
-# TODO(smcginnis) update this once six has support for collections.abc
-# (https://github.com/benjaminp/six/pull/241) or clean up once we drop py2.7.
-try:
-    from collections.abc import Hashable
-    from collections.abc import Set
-except ImportError:
-    from collections import Hashable
-    from collections import Set
-
+from collections import abc
 import itertools
 
 import six
@@ -42,7 +33,7 @@ def _merge_in(target, iterable=None, sentinel=_sentinel):
     return target
 
 
-class OrderedSet(Set, Hashable):
+class OrderedSet(abc.Set, abc.Hashable):
     """A read-only hashable set that retains insertion/initial ordering.
 
     It should work in all existing places that ``frozenset`` is used.
