@@ -15,83 +15,152 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections.abc
 import contextlib
 import datetime
-import errno
 import inspect
 import os
 import re
+import socket
 import sys
 import threading
-import time
 import types
 
+import enum
 from oslo_serialization import jsonutils
+from oslo_serialization import msgpackutils
+from oslo_utils import encodeutils
 from oslo_utils import importutils
 from oslo_utils import netutils
 from oslo_utils import reflection
 import six
-from six.moves import map as compat_map
-from six.moves import range as compat_range
 
 from taskflow.types import failure
-from taskflow.types import notifier
-from taskflow.utils import deprecation
 
 
+UNKNOWN_HOSTNAME = "<unknown>"
 NUMERIC_TYPES = six.integer_types + (float,)
 
 # NOTE(imelnikov): regular expression to get scheme from URI,
 # see RFC 3986 section 3.1
 _SCHEME_REGEX = re.compile(r"^([A-Za-z][A-Za-z0-9+.-]*):")
 
-_MONOTONIC_LOCATIONS = tuple([
-    # The built-in/expected location in python3.3+
-    'time.monotonic',
-    # NOTE(harlowja): Try to use the pypi module that provides this
-    # functionality for older versions of python less than 3.3 so that
-    # they to can benefit from better timing...
-    #
-    # See: http://pypi.python.org/pypi/monotonic
-    'monotonic.monotonic',
-])
+
+class StrEnum(str, enum.Enum):
+    """An enumeration that is also a string and can be compared to strings."""
+
+    def __new__(cls, *args, **kwargs):
+        for a in args:
+            if not isinstance(a, str):
+                raise TypeError("Enumeration '%s' (%s) is not"
+                                " a string" % (a, type(a).__name__))
+        return super(StrEnum, cls).__new__(cls, *args, **kwargs)
 
 
-def find_monotonic(allow_time_time=False):
-    """Tries to find a monotonic time providing function (and returns it)."""
-    for import_str in _MONOTONIC_LOCATIONS:
-        mod_str, _sep, attr_str = import_str.rpartition('.')
-        mod = importutils.try_import(mod_str)
-        if mod is None:
-            continue
-        func = getattr(mod, attr_str, None)
-        if func is not None:
-            return func
-    # Finally give up and use time.time (which isn't monotonic)...
-    if allow_time_time:
-        return time.time
+class StringIO(six.StringIO):
+    """String buffer with some small additions."""
+
+    def write_nl(self, value, linesep=os.linesep):
+        self.write(value)
+        self.write(linesep)
+
+
+class BytesIO(six.BytesIO):
+    """Byte buffer with some small additions."""
+
+    def reset(self):
+        self.seek(0)
+        self.truncate()
+
+
+def get_hostname(unknown_hostname=UNKNOWN_HOSTNAME):
+    """Gets the machines hostname; if not able to returns an invalid one."""
+    try:
+        hostname = socket.getfqdn()
+        if not hostname:
+            return unknown_hostname
+        else:
+            return hostname
+    except socket.error:
+        return unknown_hostname
+
+
+def match_type(obj, matchers):
+    """Matches a given object using the given matchers list/iterable.
+
+    NOTE(harlowja): each element of the provided list/iterable must be
+    tuple of (valid types, result).
+
+    Returns the result (the second element of the provided tuple) if a type
+    match occurs, otherwise none if no matches are found.
+    """
+    for (match_types, match_result) in matchers:
+        if isinstance(obj, match_types):
+            return match_result
     else:
         return None
 
 
+def countdown_iter(start_at, decr=1):
+    """Generator that decrements after each generation until <= zero.
+
+    NOTE(harlowja): we can likely remove this when we can use an
+    ``itertools.count`` that takes a step (on py2.6 which we still support
+    that step parameter does **not** exist and therefore can't be used).
+    """
+    if decr <= 0:
+        raise ValueError("Decrement value must be greater"
+                         " than zero and not %s" % decr)
+    while start_at > 0:
+        yield start_at
+        start_at -= decr
+
+
+def extract_driver_and_conf(conf, conf_key):
+    """Common function to get a driver name and its configuration."""
+    if isinstance(conf, six.string_types):
+        conf = {conf_key: conf}
+    maybe_uri = conf[conf_key]
+    try:
+        uri = parse_uri(maybe_uri)
+    except (TypeError, ValueError):
+        return (maybe_uri, conf)
+    else:
+        return (uri.scheme, merge_uri(uri, conf.copy()))
+
+
+def reverse_enumerate(items):
+    """Like reversed(enumerate(items)) but with less copying/cloning..."""
+    for i in countdown_iter(len(items)):
+        yield i - 1, items[i - 1]
+
+
 def merge_uri(uri, conf):
     """Merges a parsed uri into the given configuration dictionary.
 
-    Merges the username, password, hostname, and query params of a uri into
-    the given configuration (it does not overwrite the configuration keys if
-    they already exist) and returns the adjusted configuration.
+    Merges the username, password, hostname, port, and query parameters of
+    a URI into the given configuration dictionary (it does **not** overwrite
+    existing configuration keys if they already exist) and returns the merged
+    configuration.
 
     NOTE(harlowja): does not merge the path, scheme or fragment.
     """
-    for (k, v) in [('username', uri.username), ('password', uri.password)]:
-        if not v:
-            continue
-        conf.setdefault(k, v)
-    if uri.hostname:
-        hostname = uri.hostname
-        if uri.port is not None:
-            hostname += ":%s" % (uri.port)
-        conf.setdefault('hostname', hostname)
+    uri_port = uri.port
+    specials = [
+        ('username', uri.username, lambda v: bool(v)),
+        ('password', uri.password, lambda v: bool(v)),
+        # NOTE(harlowja): A different check function is used since 0 is
+        # false (when bool(v) is applied), and that is a valid port...
+        ('port', uri_port, lambda v: v is not None),
+    ]
+    hostname = uri.hostname
+    if hostname:
+        if uri_port is not None:
+            hostname += ":%s" % (uri_port)
+        specials.append(('hostname', hostname, lambda v: bool(v)))
+    for (k, v, is_not_empty_value_func) in specials:
+        if is_not_empty_value_func(v):
+            conf.setdefault(k, v)
     for (k, v) in six.iteritems(uri.params()):
         conf.setdefault(k, v)
     return conf
@@ -162,6 +231,23 @@ def parse_uri(uri):
     return netutils.urlsplit(uri)
 
 
+def disallow_when_frozen(excp_cls):
+    """Frozen checking/raising method decorator."""
+
+    def decorator(f):
+
+        @six.wraps(f)
+        def wrapper(self, *args, **kwargs):
+            if self.frozen:
+                raise excp_cls()
+            else:
+                return f(self, *args, **kwargs)
+
+        return wrapper
+
+    return decorator
+
+
 def clamp(value, minimum, maximum, on_clamped=None):
     """Clamps a value to ensure its >= minimum and <= maximum."""
     if minimum > maximum:
@@ -183,45 +269,31 @@ def fix_newlines(text, replacement=os.linesep):
     return replacement.join(text.splitlines())
 
 
-def binary_encode(text, encoding='utf-8'):
-    """Converts a string of into a binary type using given encoding.
+def binary_encode(text, encoding='utf-8', errors='strict'):
+    """Encodes a text string into a binary string using given encoding.
 
-    Does nothing if text not unicode string.
+    Does nothing if data is already a binary string (raises on unknown types).
     """
     if isinstance(text, six.binary_type):
         return text
-    elif isinstance(text, six.text_type):
-        return text.encode(encoding)
     else:
-        raise TypeError("Expected binary or string type not '%s'" % type(text))
+        return encodeutils.safe_encode(text, encoding=encoding,
+                                       errors=errors)
 
 
-def binary_decode(data, encoding='utf-8'):
-    """Converts a binary type into a text type using given encoding.
+def binary_decode(data, encoding='utf-8', errors='strict'):
+    """Decodes a binary string into a text string using given encoding.
 
-    Does nothing if data is already unicode string.
+    Does nothing if data is already a text string (raises on unknown types).
     """
-    if isinstance(data, six.binary_type):
-        return data.decode(encoding)
-    elif isinstance(data, six.text_type):
+    if isinstance(data, six.text_type):
         return data
     else:
-        raise TypeError("Expected binary or string type not '%s'" % type(data))
+        return encodeutils.safe_decode(data, incoming=encoding,
+                                       errors=errors)
 
 
-def decode_json(raw_data, root_types=(dict,)):
-    """Parse raw data to get JSON object.
-
-    Decodes a JSON from a given raw data binary and checks that the root
-    type of that decoded object is in the allowed set of types (by
-    default a JSON object/dict should be the root type).
-    """
-    try:
-        data = jsonutils.loads(binary_decode(raw_data))
-    except UnicodeDecodeError as e:
-        raise ValueError("Expected UTF-8 decodable data: %s" % e)
-    except ValueError as e:
-        raise ValueError("Expected JSON decodable data: %s" % e)
+def _check_decoded_type(data, root_types=(dict,)):
     if root_types:
         if not isinstance(root_types, tuple):
             root_types = tuple(root_types)
@@ -236,6 +308,40 @@ def decode_json(raw_data, root_types=(dict,)):
     return data
 
 
+def decode_msgpack(raw_data, root_types=(dict,)):
+    """Parse raw data to get decoded object.
+
+    Decodes a msgback encoded 'blob' from a given raw data binary string and
+    checks that the root type of that decoded object is in the allowed set of
+    types (by default a dict should be the root type).
+    """
+    try:
+        data = msgpackutils.loads(raw_data)
+    except Exception as e:
+        # TODO(harlowja): fix this when msgpackutils exposes the msgpack
+        # exceptions so that we can avoid catching just exception...
+        raise ValueError("Expected msgpack decodable data: %s" % e)
+    else:
+        return _check_decoded_type(data, root_types=root_types)
+
+
+def decode_json(raw_data, root_types=(dict,)):
+    """Parse raw data to get decoded object.
+
+    Decodes a JSON encoded 'blob' from a given raw data binary string and
+    checks that the root type of that decoded object is in the allowed set of
+    types (by default a dict should be the root type).
+    """
+    try:
+        data = jsonutils.loads(binary_decode(raw_data))
+    except UnicodeDecodeError as e:
+        raise ValueError("Expected UTF-8 decodable data: %s" % e)
+    except ValueError as e:
+        raise ValueError("Expected JSON decodable data: %s" % e)
+    else:
+        return _check_decoded_type(data, root_types=root_types)
+
+
 class cachedproperty(object):
     """A *thread-safe* descriptor property that is only evaluated once.
 
@@ -249,8 +355,11 @@ class cachedproperty(object):
     cached property would be stored under '_get_thing' in the self object
     after the first call to 'get_thing' occurs.
     """
-    def __init__(self, fget):
-        self._lock = threading.RLock()
+    def __init__(self, fget=None, require_lock=True):
+        if require_lock:
+            self._lock = threading.RLock()
+        else:
+            self._lock = None
         # If a name is provided (as an argument) then this will be the string
         # to place the cached attribute under if not then it will be the
         # function itself to be wrapped into a property.
@@ -264,10 +373,12 @@ class cachedproperty(object):
             self.__doc__ = None
 
     def __call__(self, fget):
-        # If __init__ received a string then this will be the function to be
-        # wrapped as a property (if __init__ got a function then this will not
-        # be called).
+        # If __init__ received a string or a lock boolean then this will be
+        # the function to be wrapped as a property (if __init__ got a
+        # function then this will not be called).
         self._fget = fget
+        if not self._attr_name:
+            self._attr_name = "_%s" % (fget.__name__)
         self.__doc__ = getattr(fget, '__doc__', None)
         return self
 
@@ -286,13 +397,17 @@ class cachedproperty(object):
         if hasattr(instance, self._attr_name):
             return getattr(instance, self._attr_name)
         else:
-            with self._lock:
-                try:
-                    return getattr(instance, self._attr_name)
-                except AttributeError:
-                    value = self._fget(instance)
-                    setattr(instance, self._attr_name, value)
-                    return value
+            if self._lock is not None:
+                self._lock.acquire()
+            try:
+                return getattr(instance, self._attr_name)
+            except AttributeError:
+                value = self._fget(instance)
+                setattr(instance, self._attr_name, value)
+                return value
+            finally:
+                if self._lock is not None:
+                    self._lock.release()
 
 
 def millis_to_datetime(milliseconds):
@@ -332,41 +447,6 @@ def sequence_minus(seq1, seq2):
     return result
 
 
-def get_duplicate_keys(iterable, key=None):
-    if key is not None:
-        iterable = compat_map(key, iterable)
-    keys = set()
-    duplicates = set()
-    for item in iterable:
-        if item in keys:
-            duplicates.add(item)
-        keys.add(item)
-    return duplicates
-
-
-class ExponentialBackoff(object):
-    """An iterable object that will yield back an exponential delay sequence.
-
-    This objects provides for a configurable exponent, count of numbers
-    to generate, and a maximum number that will be returned. This object may
-    also be iterated over multiple times (yielding the same sequence each
-    time).
-    """
-    def __init__(self, count, exponent=2, max_backoff=3600):
-        self.count = max(0, int(count))
-        self.exponent = exponent
-        self.max_backoff = max(0, int(max_backoff))
-
-    def __iter__(self):
-        if self.count <= 0:
-            raise StopIteration()
-        for i in compat_range(0, self.count):
-            yield min(self.exponent ** i, self.max_backoff)
-
-    def __str__(self):
-        return "ExponentialBackoff: %s" % ([str(v) for v in self])
-
-
 def as_int(obj, quiet=False):
     """Converts an arbitrary value into a integer."""
     # Try "2" -> 2
@@ -386,32 +466,6 @@ def as_int(obj, quiet=False):
     return obj
 
 
-# Taken from oslo-incubator file-utils but since that module pulls in a large
-# amount of other files it does not seem so useful to include that full
-# module just for this function.
-def ensure_tree(path):
-    """Create a directory (and any ancestor directories required).
-
-    :param path: Directory to create
-    """
-    try:
-        os.makedirs(path)
-    except OSError as e:
-        if e.errno == errno.EEXIST:
-            if not os.path.isdir(path):
-                raise
-        else:
-            raise
-
-
-Failure = deprecation.moved_class(failure.Failure, 'Failure', __name__,
-                                  version="0.6", removal_version="?")
-
-
-Notifier = deprecation.moved_class(notifier.Notifier, 'Notifier', __name__,
-                                   version="0.6", removal_version="?")
-
-
 @contextlib.contextmanager
 def capture_failure():
     """Captures the occurring exception and provides a failure object back.
@@ -458,3 +512,30 @@ def capture_failure():
         raise RuntimeError("No active exception is being handled")
     else:
         yield failure.Failure(exc_info=exc_info)
+
+
+def is_iterable(obj):
+    """Tests an object to to determine whether it is iterable.
+
+    This function will test the specified object to determine whether it is
+    iterable. String types (both ``str`` and ``unicode``) are ignored and will
+    return False.
+
+    :param obj: object to be tested for iterable
+    :return: True if object is iterable and is not a string
+    """
+    return (not isinstance(obj, six.string_types) and
+            isinstance(obj, collections.abc.Iterable))
+
+
+def safe_copy_dict(obj):
+    """Copy an existing dictionary or default to empty dict...
+
+    This will return a empty dict if given object is falsey, otherwise it
+    will create a dict of the given object (which if provided a dictionary
+    object will make a shallow copy of that object).
+    """
+    if not obj:
+        return {}
+    # default to a shallow copy to avoid most ownership issues
+    return dict(obj)
